package org.telegram.telegrambots.meta.generics;

import java.util.ArrayList;
import java.util.List;
import org.telegram.telegrambots.meta.api.methods.BotApiMethod;
import org.telegram.telegrambots.meta.api.methods.updates.SetWebhook;
import org.telegram.telegrambots.meta.api.objects.Update;
import org.telegram.telegrambots.meta.exceptions.TelegramApiException;
import org.telegram.telegrambots.meta.exceptions.TelegramApiRequestException;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Callback to handle updates.
 */
public interface WebhookBot extends TelegramBot {
    /**
     * This method is called when receiving updates via webhook
     * @param update Update received
     */
    BotApiMethod<?> onWebhookUpdateReceived(Update update);

    /**
     * This method is called when receiving multiple updates via webhook
     * @param updates Updates received
     */
    default List<BotApiMethod<?>> onWebhookUpdatesReceived(List<Update> updates) {
        List<BotApiMethod<?>> results = new ArrayList<>();
        for (Update update : updates) {
            BotApiMethod<?> newResult = onWebhookUpdateReceived(update);
            if (newResult != null) {
                results.add(newResult);
            }
        }
        return results;
    }

    /**
     * Execute setWebhook method to set up the url of the webhook
     * @throws TelegramApiRequestException In case of error executing the request
     */
    void setWebhook(SetWebhook setWebhook) throws TelegramApiException;

    /**
     * Gets in the url for the webhook
     * @return path in the url
     */
    String getBotPath();
}
