package org.telegram.telegrambots.updatesreceivers;

import java.util.OptionalInt;
import java.util.StringJoiner;

public class TooManyRequestsException extends RuntimeException {

    private final int processedRequestsCount;

    public TooManyRequestsException(int processedRequestsCount) {
        if (processedRequestsCount < 0) {
            throw new ArrayIndexOutOfBoundsException(processedRequestsCount);
        }
        this.processedRequestsCount = processedRequestsCount;
    }

    public TooManyRequestsException() {
        this.processedRequestsCount = -1;
    }

    public OptionalInt getProcessedRequestsCount() {
        if (processedRequestsCount == -1) {
            return OptionalInt.empty();
        } else {
            return OptionalInt.of(processedRequestsCount);
        }
    }

    @Override
    public String getMessage() {
        return "Too many requests. " + processedRequestsCount + " requests have been processed";
    }
}
